<?php

// Show all PHP errors
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

global $config;
require_once('config.php');

$testsRun       = 0;
$testsPassed    = [];
$testsFailed    = [];

// Load the test cases from CSV
$testMap = array_map('str_getcsv', file($config['testCaseFile']));

// Remove the header row if it exists
if ($testMap[0][0] == 'CDC_Test_ID') {
    array_shift($testMap);
    array_shift($testMap);
}

echo "Endpoint: " . $config['endpoint'] . "</br>";
echo "Testing " . count($testMap) . " test cases...";
// Remove all except for the first 1
//$testMap = array_slice($testMap, 0, 10);

// Remove all except where key 0 is 2013-0017
//$testMap = array_filter($testMap, function($a) { return $a[0] == '2013-0016'; });

// Run each test case
foreach ($testMap as $test) {
    $testData                               = [];
    $testData['id']                         = $test[0];
    $testData['patient']['dob']             = $test[2];
    $testData['patient']['gender']          = $test[3];
    $testData['group']['vaccineGroup']      = $test[54];
    $testData['group']['expectedOutcome']   = $test[7];
    $testData['group']['recommendedDate']   = $test[52];
    $testData['assessmentDate']             = $test[55];
    $testData['recommendedDose']            = $test[50];
    
    // Get Administrations from the test case
    $admin = [];
    for($i = 8; $i < 50; $i++) {
        $admin[] = $test[$i];
    }

    // Each test case has a different number of administrations, so we need to remove the empty ones - each administration is 6 columns
    $admin = array_chunk($admin, 6);
    $admin = array_filter($admin, function($a) { return $a[0] != ''; });

    // Several dates are in mm/dd/YYYY format, but we need to convert them to YYYYmmdd
    $testData['patient']['dob']             = date('Ymd', strtotime($testData['patient']['dob']));
    $testData['group']['recommendedDate']   = date('Ymd', strtotime($testData['group']['recommendedDate']));
    $testData['assessmentDate']             = date('Ymd', strtotime($testData['assessmentDate']));

    // Build the XML request
    $xmlPerson = '<ns:personDetails personId="' . $testData['id'] . '" evaluationDate="' . $testData['assessmentDate'] . '">
<ns:dateOfBirth>' . $testData['patient']['dob'] . '</ns:dateOfBirth>
<ns:gender>' . $testData['patient']['gender'] . '</ns:gender>';

    $xmlAdmin = '';

    foreach ($admin as $a) {
        $xmlAdmin .= '<ns:vaccination codeType="CVX">
<ns:vaccCode>' . $a[2] . '</ns:vaccCode>
<ns:vaccDate>' . date('Ymd', strtotime($a[0])) . '</ns:vaccDate>
</ns:vaccination>';
    }

    $xmlPerson .= $xmlAdmin . '</ns:personDetails>';

    $xml = '<soapenv:Envelope xmlns:com="http://saf.stchome.com/common" xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns="http://saf.stchome.com/messages/get/1.3">
<soapenv:Header/>
<soapenv:Body>
<ns:GetForecastRequest>' . $xmlPerson . '</ns:GetForecastRequest>
</soapenv:Body>
</soapenv:Envelope>';

    // remove all linebreaks from the XML
    $xml = str_replace("\n", '', $xml);

    // Send the request to the STC endpoint
    $ch = curl_init($config['endpoint']);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $response = curl_exec($ch);
    curl_close($ch);

    $result = curl_exec($ch);

    // We need to parse the response to check the results
    $result = ingestSTC($result)['soapenv_Body']['ns_GetForecastResponse']['ns_responses'];

    $passing = true;
    $problem = "";
    
    // Check the forecasted group
    $resultGroup = groupToSTCGroup($testData['group']['vaccineGroup']);

    // If it shouldn't be there...
    if (in_array($testData['recommendedDose'], ["", "-"])) {
        foreach($result['ns_forecastDetails'] as $group) {
            if ($group['ns_familyCode'] == $resultGroup) {
                if ($testData['recommendedDose'] == "-") {
                    $passing = false;
                    $problem .= " [Group appeared recommended when it shouldn't have been] ";
                } else {
                    $problem .= " [CONDITIONAL: appeared on forecast while temporarily completed] ";
                }
            }
        }
    } else {
        // If it should be there...

        $found = false;

        foreach($result['ns_forecastDetails'] as $group) {
            if ($group['ns_familyCode'] == $resultGroup) {
                $found = true;

                // Check the recommended date
                if (date('Ymd', strtotime($group['ns_recommendedDate'])) != $testData['group']['recommendedDate']) {
                    $passing = false;
                    $problem .= " [Recommended date did not match] ";
                }
            }
        }

        if (!$found) {
            $passing = false;
            $problem .= " [Group was not recommended when it should have been] ";
        }
    }

    $vaccinationsReturned = isset($result['ns_personDetails']['ns_vaccination']) ? $result['ns_personDetails']['ns_vaccination'] : [];
    // We always want this to be an array, even if there's only one
    if (count($vaccinationsReturned)) {
        $keys = array_keys($vaccinationsReturned);
        $isList = is_int($keys[0]);

        if (!$isList) {
            $newReturned = [];
            $newReturned[] = $vaccinationsReturned;
            $vaccinationsReturned = $newReturned;
        }

        // Check the administered vaccines
        foreach($admin as $vacc) {
            $found = false;

            foreach($vaccinationsReturned as $indVacc) {
                
                if ($indVacc['ns_vaccCode'] == $vacc[2] && $indVacc['ns_vaccDate'] == date('Ymd', strtotime($vacc[0]))) {
                    $found = true;

                    $expectedValidity = valid($vacc[4]);
                    
                    // Check if $indVacc['message'] is an array
                    if (isset($indVacc['ns_message'])) {
                        $keys = array_keys($indVacc['ns_message']);
                        $isList = is_int($keys[0]);

                        if (!$isList) {
                            $newReturned = [];
                            $newReturned[] = $indVacc['ns_message'];
                            $indVacc['ns_message'] = $newReturned;
                        }
                        
                        foreach($indVacc['ns_message'] as $message) {
                            if ($message['@attributes']['messageType'] !== $expectedValidity) {
                                $passing = false;
                                $problem .= " [Dose of {$vacc[2]} on {$vacc[0]} returned a type of {$message['@attributes']['messageType']} - Expected {$expectedValidity}] ";
                            }
                        }
                    } else {
                        $passing = false;
                        $problem .= " [Dose of {$vacc[2]} on {$vacc[0]} returned no message] ";
                    }
                }
            }

            if (!$found) {
                $passing = false;
                $problem .= " [Vaccine administered at {$indVacc['ns_vaccDate']} was not returned] ";
            }
        }
    }

    if ($passing) {
        $testsPassed[] = [$testData['id'], $problem];
    } else {
        $testsFailed[] = [$testData['id'], $problem];
    }

    $testsRun++;
    // Unset the variables so we can reuse them
    unset($testData);
}

echo "<br><br>Tests run: " . $testsRun . "<br>";
echo "Tests passed: " . count($testsPassed) . "<br>";
echo "Tests failed: " . count($testsFailed) . "<br>";

// Show a percentage passing score
echo "Score: " . round(count($testsPassed) / $testsRun * 100, 2) . "%<br><br>";

// Make a table where one column is the test ID and the other is the result
$testTable = [];
foreach ($testsPassed as $test) {
    $testTable[] = [$test[0], $test[1], "Passed"];
}
foreach ($testsFailed as $test) {
    $testTable[] = [$test[0], $test[1], "Failed"];
}

// Output the table in HTML
echo "<table><tr><th>Test ID</th><th>Result</th><th>Problem</th></tr>";
foreach ($testTable as $test) {
    echo "<tr><td>{$test[0]}</td><td>{$test[2]}</td><td>{$test[1]}</td></tr>";
}
echo "</table>";

function ingestSTC($data) {

    // Change instances of "SOAP-ENV" to "soapenv"
    $data = str_replace('SOAP-ENV', 'soapenv', $data);

    $plainXML = mungXML( trim($data) );
    $arrayResult = json_decode(json_encode(SimpleXML_Load_String($plainXML, 'SimpleXMLElement', LIBXML_NOCDATA)), true);

    return $arrayResult;
}

function mungXML($xml)
{
    $obj = SimpleXML_Load_String($xml);
    if ($obj === FALSE) return $xml;

    // GET NAMESPACES, IF ANY
    $nss = $obj->getNamespaces(TRUE);
    if (empty($nss)) return $xml;

    // CHANGE ns: INTO ns_
    $nsm = array_keys($nss);
    foreach ($nsm as $key)
    {
        // A REGULAR EXPRESSION TO MUNG THE XML
        $rgx
        = '#'               // REGEX DELIMITER
        . '('               // GROUP PATTERN 1
        . '\<'              // LOCATE A LEFT WICKET
        . '/?'              // MAYBE FOLLOWED BY A SLASH
        . preg_quote($key)  // THE NAMESPACE
        . ')'               // END GROUP PATTERN
        . '('               // GROUP PATTERN 2
        . ':{1}'            // A COLON (EXACTLY ONE)
        . ')'               // END GROUP PATTERN
        . '#'               // REGEX DELIMITER
        ;
        // INSERT THE UNDERSCORE INTO THE TAG NAME
        $rep
        = '$1'          // BACKREFERENCE TO GROUP 1
        . '_'           // LITERAL UNDERSCORE IN PLACE OF GROUP 2
        ;
        // PERFORM THE REPLACEMENT
        $xml =  preg_replace($rgx, $rep, $xml);
    }

    return $xml;

}

function groupToSTCGroup($str) {
    $groups = [
        'DTAP'      => '1',
        'FLU'       => '10',
        'HepA'      => '9',
        'HepB'      => '4',
        'HIB'       => '2',
        'HPV'       => '21',
        'MCV'       => '7',
        'MMR'       => '5',
        'PCV'       => '11',
        'POL'       => '3',
        'ROTA'      => '20',
        'VAR'       => '6',
        'ZOSTER'    => '22',
        'COVID-19'  => '33'
    ];

    return $groups[$str];
}

function valid($input) {
    if ($input == "Valid") {
        return "VALID-VACC";
    }

    return "INVALID-VACC";
}