<?php
// Handle configuration
global $config;
require_once('configuration/config.php');

// Write all PHP errors to error.log
ini_set('log_errors', 1);
ini_set('error_log', 'error.log');

// Set spoof STC headers
header('Content-Type: text/xml;charset=utf-8');
header('SOAPAction: ""');

// Check if the client can accept gzip encoding
$gzip = false;
if (strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') !== false) {
    header('Content-Encoding: gzip');
    $gzip = true;
}

$error = [];

// Get the data
$data = file_get_contents('php://input');

if (!empty($data)) {
    // Consume the data
    require('util/ingest.php');
    $parsed         = ingestSTC($data);

    // Build our encoded payload
    require('util/construct_request.php');
    $request        = constructRequest($parsed);

    // Send the request
    require('util/send_request.php');
    $response       = sendRequest($request['payload']);

    // Since HLN ICE responds in a SOAP envelope, we can utilize the STC ingestion technique to return the data since we're just looking for the encoded payload
    // Once we have the encoded payload, we can decode it and process it again (I heard you liked SOAP, so I put SOAP in your SOAP)
    $parsedResponse = ingestSTC($response)['soap_Body']['ns2_evaluateAtSpecifiedTimeResponse']['evaluationResponse']['finalKMEvaluationResponse']['kmEvaluationResultData']['data']['base64EncodedPayload'];
    $parsedResponse = base64_decode($parsedResponse);
    $parsedResponse = ingestSTC($parsedResponse);

    // Now let's have fun with parsing the ICE payload...
    require('util/respond.php');
    $parsedResponse = parseResponse($parsedResponse['vmrOutput'], $request['futureInfo']);

    if ($gzip) {
        $parsedResponse = gzencode($parsedResponse);
    }

    // Return the response to the client
    echo $parsedResponse;

} else {
    $error[] = 'No input data received';
}


?>
