<?php
global $config;

function parseResponse($payload, $info) {
    global $config;

    // Build the response from ICE into a universal format
    $map = [];

    // Handle metadata points
    $map['metadata']['evaluationDate']  = $info['evaluationDate'];
    $map['metadata']['personId']        = $info['personId'];
    $map['metadata']['ID']              = [$payload['patient']['id']['@attributes']['root'], $payload['patient']['id']['@attributes']['extension']];  

    // Handle patient information
    $map['patient']['dateOfBirth']      = substr($payload['patient']['demographics']['birthTime']['@attributes']['value'], 0, 8);
    $map['patient']['gender']           = $payload['patient']['demographics']['gender']['@attributes']['code'];

    $clinicalStatements                 = $payload['patient']['clinicalStatements'];

    //
    // Forecast
    //
    $map['forecast']                    = [];
    $map['finishedSeries']              = [];

    // We always want the forecast to be an array, even if there is only one (or none) forecasted vaccine
    $keys = array_keys($clinicalStatements['substanceAdministrationProposals']['substanceAdministrationProposal']);
    $isList = is_int($keys[0]);

    if (!$isList) {
        $newClinical = [];
        $newClinical[] = $clinicalStatements['substanceAdministrationProposals']['substanceAdministrationProposal'];
        $clinicalStatements['substanceAdministrationProposals']['substanceAdministrationProposal'] = $newClinical;
    }

    foreach ($clinicalStatements['substanceAdministrationProposals']['substanceAdministrationProposal'] as $forecasted) {
        // Check that the forecasted vaccine is recommended or due in the future

        // We'll use this map to determine if we should deliver this forecast or not (if it's not recommended or due in the future, we won't deliver it)
        $forecastMap = ['RECOMMENDED', 'FUTURE_RECOMMENDED'];

        if (in_array($forecasted['relatedClinicalStatement']['observationResult']['observationValue']['concept']['@attributes']['code'], $forecastMap)) {

            $defaultMaxValid    = date('Ymd', strtotime($map['patient']['dateOfBirth'] . ' +120 years'));
            $familyCode         = $forecasted['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code'];
            $cvx                = ($forecasted['substance']['substanceCode']['@attributes']['code'] == $forecasted['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code']) ? 0 : $forecasted['substance']['substanceCode']['@attributes']['code'];
            $gracePeriod        = " -" . strval($config['OC']['gracePeriod']) . " days";
            $map['forecast'][]  =   [
                                        'forecastCVXCode'   => $cvx,
                                        'familyCode'        => $familyCode,
                                        'doseNumber'        => 1, // This ends up being modified later based on previously administered vaccines
                                        'maxAllowableDate'  => isset($forecasted['validAdministrationTimeInterval']['@attributes']['high']) ? substr($forecasted['validAdministrationTimeInterval']['@attributes']['high'], 0, 8) : $defaultMaxValid,
                                        'minAllowableDate'  => isset($forecasted['validAdministrationTimeInterval']['@attributes']['low']) ? date('Ymd', strtotime(substr($forecasted['validAdministrationTimeInterval']['@attributes']['low'], 0, 8) . $gracePeriod)) : $map['metadata']['evaluationDate'],
                                        'recommendedDate'   => substr($forecasted['proposedAdministrationTimeInterval']['@attributes']['low'], 0, 8),
                                        'pastDueDate'       => isset($forecasted['proposedAdministrationTimeInterval']['@attributes']['high']) ? date('Ymd', strtotime(substr($forecasted['proposedAdministrationTimeInterval']['@attributes']['high'], 0, 8))) : $defaultMaxValid,
                                    ];
        }

        if (isset($forecasted['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['code']) && in_array($forecasted['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['code'], ["COMPLETE", "COMPLETE_HIGH_RISK"])) {
            // Add the family to the finished series list
            $map['finishedSeries'][] = $forecasted['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code'];
        }
    }

    //
    // Administered Vaccines
    //

    $map['administeredVaccines']        = [];

    if (isset($clinicalStatements['substanceAdministrationEvents'])) {
        // We always want the administered vaccines to be an array, even if there is only one (or none) administered vaccine
        $keys = array_keys($clinicalStatements['substanceAdministrationEvents']['substanceAdministrationEvent']);
        $isList = is_int($keys[0]);

        if (!$isList) {
            $newClinical = [];
            $newClinical[] = $clinicalStatements['substanceAdministrationEvents']['substanceAdministrationEvent'];
            $clinicalStatements['substanceAdministrationEvents']['substanceAdministrationEvent'] = $newClinical;
        }

        foreach ($clinicalStatements['substanceAdministrationEvents']['substanceAdministrationEvent'] as $vaccine) {

            // Combination vaccines will cause multiple nodes within one vaccine's relatedClinicalStatements - if that's the case, we need to return multiple message segments
            $multi = false;
            if (isset($vaccine['relatedClinicalStatement'][0])) {
                $multi = true;
            }
            
            // Add each vaccination to the map - these are the values we need to output, however, they need to be mapped to STC codes
            $map['administeredVaccines'][] =    [
                                                    'vaccCode'          => $vaccine['substance']['substanceCode']['@attributes']['code'],
                                                    'vaccDate'          => isset($vaccine['administrationTimeInterval']['@attributes']['low']) ? substr($vaccine['administrationTimeInterval']['@attributes']['low'], 0, 8) : "",
                                                    'messages'          => []
                                                ];                                    

            if (!$multi) {
                $map['administeredVaccines'][count($map['administeredVaccines']) - 1]['messages'][] = [
                                                                                                        'familyCode'        => isset($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code']) ? trim($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code']) : '',
                                                                                                        'doseNumber'        => isset($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['doseNumber']['@attributes']['value']) ? $vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['doseNumber']['@attributes']['value'] : 1,
                                                                                                        'seriesComplete'    => in_array($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code'], $map['finishedSeries']) ? true : false,
                                                                                                        'messageType'       => isset($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['code']) ? trim($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['code']) : '',
                                                                                                        'message'           => isset($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['displayName']) ? trim($vaccine['relatedClinicalStatement']['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['displayName']) : ''
                                                                                                    ];
            } else {
                foreach ($vaccine['relatedClinicalStatement'] as $message) {
                    $map['administeredVaccines'][count($map['administeredVaccines']) - 1]['messages'][] =   [
                                                                                                                'familyCode'        => isset($message['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code']) ? trim($message['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code']) : '',
                                                                                                                'doseNumber'        => isset($message['substanceAdministrationEvent']['doseNumber']['@attributes']['value']) ? $message['substanceAdministrationEvent']['doseNumber']['@attributes']['value'] : 1,
                                                                                                                'seriesComplete'    => in_array($message['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['observationFocus']['@attributes']['code'], $map['finishedSeries']) ? true : false,
                                                                                                                'messageType'       => isset($message['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['code']) ? trim($message['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['code']) : '',
                                                                                                                'message'           => isset($message['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['displayName']) ? trim($message['substanceAdministrationEvent']['relatedClinicalStatement']['observationResult']['interpretation']['@attributes']['displayName']) : ''
                                                                                                            ];
                }
            }
        }
    }
    
    // 
    // We need to apply the dose number to the forecasted vaccines, so we'll do that by checking the administered vaccines
    //

    $map['series'] = [];

    foreach ($map['administeredVaccines'] as $vaccine) {

        foreach ($vaccine['messages'] as $individualVacc) {
            // Check if the vaccine is in the series map
            if (!isset($map['series'][$individualVacc['familyCode']])) {
                // Add the series to the map
                $map['series'][$individualVacc['familyCode']] = intval($individualVacc['doseNumber']);
            }

            // Check if the vaccine is in the series map
            if (intval($map['series'][$individualVacc['familyCode']]) < intval($individualVacc['doseNumber'])) {
                // Make sure this is the number in there now
                $map['series'][$individualVacc['familyCode']] = intval($individualVacc['doseNumber']);
            }
        }
    }

    // Loop through the forecasted vaccines and add the dose number to the map
    foreach ($map['forecast'] as $key => $vaccine) {
        // Check if the vaccine is in the series map
        if (isset($map['series'][$vaccine['familyCode']])) {
            // Add the dose number to the map
            $map['forecast'][$key]['doseNumber'] = $map['series'][$vaccine['familyCode']] + 1;
        }
    }

    // Supress forecasted vaccines that are in $config['OC']['suppressFirstDose']
    foreach ($map['forecast'] as $key => $vaccine) {
        if (in_array(intval($vaccine['familyCode']), $config['OC']['suppressFirstDose']) && intval($vaccine['doseNumber']) == 1) {
            unset($map['forecast'][$key]);
        }
    }

    // Find how old the patient is in years
    $dob    = $map['patient']['dateOfBirth'];
    $eval   = $map['metadata']['evaluationDate'];
    $dob    = DateTime::createFromFormat('Ymd', $dob);
    $eval   = DateTime::createFromFormat('Ymd', $eval);
    $interval = $dob->diff($eval);
    $years = $interval->y;

    // Supress forecasted vaccines for which the patient has not met the minimum age in $config['OC']['minAge']
    $map['patient']['age'] = $years;
    foreach ($map['forecast'] as $key => $vaccine) {
        if (isset($config['OC']['minAge'][$vaccine['familyCode']]) && $years < $config['OC']['minAge'][$vaccine['familyCode']]) {
            unset($map['forecast'][$key]);
        }
    }

    // ICE has the habit of jumbling the forecast order, so we'll sort them by family code
    usort($map['forecast'], function($a, $b) {
        return $a['familyCode'] <=> $b['familyCode'];
    });

    // Convert the incompatible ICE codes to STC codes
    $convertedMap   = convertICE($map);

    // VACCINE GROUP MODIFICATIONS
    // Some vaccine groups need to be differentiated (ie, STC has multiple groups where ICE has one and likely returned a CVX code)
    // In these cases, we perform some magic to make sure the correct group is returned

    // PCV vs PPSV
    require 'util/conditional/pneumo.php';
    foreach ($convertedMap['forecast'] as $key => $vaccine) {
        if ($vaccine['familyCode'] == '18') {
            $convertedMap['forecast'][$key]['familyCode'] = pneumoGroup($vaccine['forecastCVXCode']);
        }
    }

    $output        = build($convertedMap);

    return $output;

}

function build($payload) {
    // Build the fake STC payload that will be returned to the client

    $personId = isset($payload['metadata']['personId']) ? $payload['metadata']['personId'] : "";

    // Header
    $xml = '<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Header/>
<SOAP-ENV:Body>
<ns:GetForecastResponse xmlns:ns="http://saf.stchome.com/messages/get/1.3">
<ns:responses>
<ns:personDetails evaluationDate="' . $payload['metadata']['evaluationDate'] . '" fullForecast="" personId="' . $personId . '" pretermInfant="" serologyDate="">
<ns:scheduleType xsi:nil="true" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>
<ns:dateOfBirth>' . $payload['patient']['dateOfBirth'] . '</ns:dateOfBirth>
<ns:gender>' . $payload['patient']['gender'] . '</ns:gender>';

// Add the administered vaccines
foreach ($payload['administeredVaccines'] as $vaccine) {

    $xml .= '<ns:vaccination codeType="CVX">
<ns:vaccCode>' . $vaccine['vaccCode'] . '</ns:vaccCode>
<ns:vaccDate>' . $vaccine['vaccDate'] . '</ns:vaccDate>
<ns:doseSize>1</ns:doseSize>';
foreach($vaccine['messages'] as $message) {
    $completeSeries = $message['seriesComplete'] ? "Y" : "N";
    $xml .= '<ns:message doseNumber="' . $message['doseNumber'] . '" familyCode="' . $message['familyCode'] . '" messageType="' . $message['messageType'] . '" seriesComplete="' . $completeSeries . '">
<com:message xmlns:com="http://saf.stchome.com/common">' . $message['message'] . '</com:message>
</ns:message>';
}
$xml .= '<ns:compromisedCode xsi:nil="true" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>
</ns:vaccination>';
}

$xml .= '</ns:personDetails>';

// Add the forecasted vaccines
foreach ($payload['forecast'] as $vaccine) {
    $xml .= '<ns:forecastDetails>
<ns:familyCode>' . $vaccine['familyCode'] . '</ns:familyCode>
<ns:doseNumber>' . $vaccine['doseNumber'] . '</ns:doseNumber>
<ns:maxAllowableDate>' . $vaccine['maxAllowableDate'] . '</ns:maxAllowableDate>
<ns:minAllowableDate>' . $vaccine['minAllowableDate'] . '</ns:minAllowableDate>
<ns:recommendedDate>' . $vaccine['recommendedDate'] . '</ns:recommendedDate>
<ns:pastDueDate>' . $vaccine['pastDueDate'] . '</ns:pastDueDate>
<ns:forecastCVXCode>' . $vaccine['forecastCVXCode'] . '</ns:forecastCVXCode>
<ns:doseCount>0</ns:doseCount>
<ns:booster>false</ns:booster>
</ns:forecastDetails>';
}

$xml .= '<ns:information>Forecast success.</ns:information>
<ns:releaseVersion>v1.0 OpenCast</ns:releaseVersion>
</ns:responses>
</ns:GetForecastResponse>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>';

//Remove all linebreaks from the xml
$xml = preg_replace("/\r|\n/", "", $xml);

return $xml;

}

function buildVaccineFamiliesTable() {
    // Load the csv file called vaccineFamilies.csv
    $vaccineFamilies = array_map('str_getcsv', file('translationData/vaccineFamilies.csv'));

    $vaccineFamilyTable = [];

    // Loop through the vaccine families and build the table
    foreach ($vaccineFamilies as $vaccineFamily) {
        if (intval($vaccineFamily[4]) == 1) {
            $vaccineFamilyTable[$vaccineFamily[2]] = [$vaccineFamily[0], $vaccineFamily[1]];
        }
    }

    return $vaccineFamilyTable;
}

function convertICE($map) {
    // Load up conversion tables
    $vaccineFamilies = buildVaccineFamiliesTable();

    // Loop through the forecasted vaccines and convert the ICE codes to STC codes
    foreach ($map['forecast'] as $key => $vaccine) {
        // Check if the vaccine is in the series map
        if (isset($vaccineFamilies[$vaccine['familyCode']])) {
            // Change the family code to the STC code
            $map['forecast'][$key]['familyCode'] = $vaccineFamilies[$vaccine['familyCode']][0];
        }
    }

    // Loop through the administered vaccines and convert the ICE codes to STC codes
    foreach ($map['administeredVaccines'] as $key => $vaccine) {
        foreach($vaccine['messages'] as $key2 => $individualVacc) {
        // Check if the vaccine is in the series map
            if (isset($vaccineFamilies[$individualVacc['familyCode']])) {
                // Change the family code to the STC code
                $map['administeredVaccines'][$key]['messages'][$key2]['familyCode'] = $vaccineFamilies[$individualVacc['familyCode']][0];
            }

            // Check if the vaccine's message type exists or not
            if ($map['administeredVaccines'][$key]['messages'][$key2]['messageType'] !== "") {
                // Set the message type to the default
                $map['administeredVaccines'][$key]['messages'][$key2]['messageType'] = 'INVALID-VACC';
            } else {
                // Set the message type to the default
                $map['administeredVaccines'][$key]['messages'][$key2]['messageType'] = 'VALID-VACC';
                $map['administeredVaccines'][$key]['messages'][$key2]['message'] = 'Valid Vaccination';
            }
        }
    }

    return $map;

}

?>